==> evms_2.3.4.orig.tar.gz: evms-2.3.4/ui/utils/evms_activate.c <==
/*
 *   (C) Copyright IBM Corp. 2001, 2003
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * evms_activate
 *
 * Simple command-line utility to open the engine and perform a commit, which
 * will activate all volumes.
 */

#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <libgen.h>
#include <frontend.h>

#define REQUIRED_ENGINE_API_MAJOR 9
#define REQUIRED_ENGINE_API_MINOR 0
#define REQUIRED_ENGINE_API_PATCH 0

static char * prog_name;				
static debug_level_t debug_level = -1;
static char * log_file = NULL;
static char * volume_name = NULL;
static char * node_name = NULL;
static boolean quiet = FALSE;
static boolean verbose = FALSE;

static boolean is_numeric(char * str)
{
	while (*str != '\0') {
		if ((*str < '0') || (*str > '9')) {
			return FALSE;
		}
		str++;
	}
	return TRUE;
}

static void show_help(void)
{
	printf("Usage: %s [options]\n", prog_name);
	printf("Activate all EVMS volumes.\n\n"
	       "  [-d debug_level, --debug-level debug_level]\n"
	       "     debug_level = [0-9] |\n"
	       "                   [[critical | serious | error | warning | default |\n"
	       "                     details | debug | extra | entry_exit | everything]]\n"
	       "  [-l log_file_name, --log-file log_file_name]\n"
	       "  [--devnum volume_name]\n"
	       "      print the device number for volume_name\n"
	       "  [-n node_name, --node-name node_name]\n"
	       "  [-q] quiet mode\n"
	       "  [-v] verbose mode\n"
	       "  [-h | -? | --help]\n"
	       "     display this help\n");
}

static int parse_options(int argc, char ** argv)
{
	int c, rc = 0;
	char * short_opts = "d:l:n:qvh?";
	struct option long_opts[] = { { "debug-level",  required_argument, NULL, 'd'},
				      { "log-file",     required_argument, NULL, 'l'},
				      { "devnum",	required_argument, NULL, 'D'},
				      { "node-name",    required_argument, NULL, 'n'},
				      { "help",         no_argument,       NULL, 'h'},
				      { NULL,           0,                 NULL,   0}  };

	while ((c = getopt_long(argc, argv, short_opts,
				long_opts, NULL)) != EOF) {
		switch (c) {
		case 'd':
			if (is_numeric(optarg)) {
				int level = atoi(optarg);

				if ((level < CRITICAL) ||
				    (level > EVERYTHING)) {
					fprintf(stderr, "%s is not a valid debug level.\n",
					       optarg);
					/* Display the help. */
					rc = 1;
				}
			} else if (strcasecmp(optarg, "critical") == 0) {
				debug_level = CRITICAL;
			} else if (strcasecmp(optarg, "serious") == 0) {
				debug_level = SERIOUS;
			} else if (strcasecmp(optarg, "error") == 0) {
				debug_level = ERROR;
			} else if (strcasecmp(optarg, "warning") == 0) {
				debug_level = WARNING;
			} else if (strcasecmp(optarg, "default") == 0) {
				debug_level = DEFAULT;
			} else if (strcasecmp(optarg, "details") == 0) {
				debug_level = DETAILS;
			} else if (strcasecmp(optarg, "debug") == 0) {
				debug_level = DEBUG;
			} else if (strcasecmp(optarg, "extra") == 0) {
				debug_level = EXTRA;
			} else if (strcasecmp(optarg, "entry_exit") == 0) {
				debug_level = ENTRY_EXIT;
			} else if (strcasecmp(optarg, "everything") == 0) {
				debug_level = EVERYTHING;
			} else {
				fprintf(stderr, "%s is not a valid debug "
					"level.\n", optarg);
				/* Display the help. */
				rc = EINVAL;
			}

			break;

		case 'l':
			log_file = strdup(optarg);
			break;

		case 'D':
			volume_name = strdup(optarg);
			break;

		case 'n':
			node_name = strdup(optarg);
			break;

		case 'q':
			quiet = TRUE;
			break;

		case 'v':
			verbose = TRUE;
			break;

		case 'h':
		case '?':
			/* Display the help. */
			rc = EINVAL;
			break;

		default:
			fprintf(stderr, "%s -- unrecognized option \"%c\"\n\n",
			       prog_name, c);
			/* Display the help. */
			rc = EINVAL;
			break;
		}
	}

	return rc;
}

static int message_callback(char * message_text,
			    int * answer,
			    char ** choices)
{
	if (quiet == FALSE) {
		printf(message_text);

		if (choices != NULL) {
			printf("%s: Responding with default selection \"%s\".\n",
			       prog_name, choices[*answer]);
		}
	}

	return 0;
}

static void status_callback(char * message_text)
{
	if (verbose == TRUE) {
		printf(message_text);
	}
}

static ui_callbacks_t callbacks = {
	user_message: message_callback,
	user_communication: NULL,
	progress: NULL,
	status: status_callback
};

int main(int argc, char **argv)
{
	evms_version_t engine_api;
	int rc;

	prog_name = basename(argv[0]);

	rc = parse_options(argc, argv);
	if (rc) {
		show_help();
		return rc;
	}

	evms_get_api_version(&engine_api);

	if ((engine_api.major != REQUIRED_ENGINE_API_MAJOR) ||
	    (engine_api.major == REQUIRED_ENGINE_API_MAJOR &&
	     engine_api.minor <  REQUIRED_ENGINE_API_PATCH)) {
		fprintf(stderr, "EVMS API version mismatch.\n");
		fprintf(stderr, "\tEVMS provides %d.%d.%d\n",
			engine_api.major, engine_api.minor, engine_api.patchlevel);
		fprintf(stderr, "\t%s requires %d.%d.%d\n",
			prog_name, REQUIRED_ENGINE_API_MAJOR,
			REQUIRED_ENGINE_API_MINOR, REQUIRED_ENGINE_API_PATCH);
		return 1;
	}

	rc = evms_open_engine(node_name, ENGINE_READWRITE,
			      &callbacks, debug_level, log_file);
	if (rc) {
		fprintf(stderr, "Error returned from evms_open_engine(): %s\n",
			evms_strerror(rc));
		return rc;
	}

	rc = evms_commit_changes();
	if (rc) {
		fprintf(stderr, "Error returned from evms_commit_changes(): "
			"%s\n", evms_strerror(rc));
	}

	if (volume_name) {
		object_handle_t handle;
		handle_object_info_t *info;
		int rc2, dev;

		rc2 = evms_get_object_handle_for_name(VOLUME,
						      volume_name, &handle);
		if (!rc2) {
			rc2 = evms_get_info(handle, &info);
			if (!rc2) {
				dev = (info->info.volume.dev_major << 8) +
				      info->info.volume.dev_minor;
				printf("%d\n", dev);
				evms_free(info);
			} else {
				fprintf(stderr, "Error returned from evms_get_"
					"info(): %s\n", evms_strerror(rc2));
			}
		} else {
			fprintf(stderr, "Error returned from evms_get_object_"
				"handle_for_name(): %s\n", evms_strerror(rc2));
		}
	}

	evms_close_engine();

	return rc;
}

==> evms_2.5.5.orig.tar.gz: evms-2.5.5/ui/utils/evms_activate.c <==
/*
 *   (C) Copyright IBM Corp. 2001, 2004
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * evms_activate
 *
 * Simple command-line utility to open the engine and perform a commit, which
 * will activate all volumes.
 */

#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <libgen.h>
#include <frontend.h>
#include <locale.h>

#define REQUIRED_ENGINE_API_MAJOR 10
#define REQUIRED_ENGINE_API_MINOR 0
#define REQUIRED_ENGINE_API_PATCH 0

static char * prog_name;				
static debug_level_t debug_level = -1;
static char * log_file = NULL;
static char * volume_name = NULL;
static char * node_name = NULL;
static boolean quiet = FALSE;
static boolean verbose = FALSE;

static boolean is_numeric(char * str)
{
	while (*str != '\0') {
		if ((*str < '0') || (*str > '9')) {
			return FALSE;
		}
		str++;
	}
	return TRUE;
}

static void show_help(void)
{
	printf(_("Usage: %s [options]\n"), prog_name);
	printf(_("Activate all EVMS volumes.\n\n"
		 "  [-d debug_level, --debug-level debug_level]\n"
		 "     debug_level = [0-9] |\n"
		 "                   [[critical | serious | error | warning | default |\n"
		 "                     details | entry_exit | debug | extra | everything]]\n"
		 "  [-l log_file_name, --log-file log_file_name]\n"
		 "  [--devnum volume_name]\n"
		 "      print the device number for volume_name\n"
		 "  [-n node_name, --node-name node_name]\n"
		 "  [-q] quiet mode\n"
                 "  [-v] verbose mode\n"
		 "  [-h | -? | --help]\n"
		 "     display this help\n"));
}

static int parse_options(int argc, char ** argv)
{
	int c, rc = 0;
	char * short_opts = "d:l:n:qvh?";
	struct option long_opts[] = { { "debug-level",  required_argument, NULL, 'd'},
				      { "log-file",     required_argument, NULL, 'l'},
				      { "devnum",	required_argument, NULL, 'D'},
				      { "node-name",    required_argument, NULL, 'n'},
				      { "help",         no_argument,       NULL, 'h'},
				      { NULL,           0,                 NULL,   0}  };

	while ((c = getopt_long(argc, argv, short_opts,
				long_opts, NULL)) != EOF) {
		switch (c) {
		case 'd':
			if (is_numeric(optarg)) {
				int level = atoi(optarg);

				if ((level < CRITICAL) ||
				    (level > EVERYTHING)) {
					fprintf(stderr, _("%s is not a valid debug level.\n"),
					       optarg);
					/* Display the help. */
					rc = 1;
				}
			} else if (strcasecmp(optarg, "critical") == 0) {
				debug_level = CRITICAL;
			} else if (strcasecmp(optarg, "serious") == 0) {
				debug_level = SERIOUS;
			} else if (strcasecmp(optarg, "error") == 0) {
				debug_level = ERROR;
			} else if (strcasecmp(optarg, "warning") == 0) {
				debug_level = WARNING;
			} else if (strcasecmp(optarg, "default") == 0) {
				debug_level = DEFAULT;
			} else if (strcasecmp(optarg, "details") == 0) {
				debug_level = DETAILS;
			} else if (strcasecmp(optarg, "entry_exit") == 0) {
				debug_level = ENTRY_EXIT;
			} else if (strcasecmp(optarg, "debug") == 0) {
				debug_level = DEBUG;
			} else if (strcasecmp(optarg, "extra") == 0) {
				debug_level = EXTRA;
			} else if (strcasecmp(optarg, "everything") == 0) {
				debug_level = EVERYTHING;
			} else {
				fprintf(stderr, _("%s is not a valid debug level.\n"),
						  optarg);
				/* Display the help. */
				rc = EINVAL;
			}

			break;

		case 'l':
			log_file = strdup(optarg);
			break;

		case 'D':
			volume_name = strdup(optarg);
			break;

		case 'n':
			node_name = strdup(optarg);
			break;

		case 'q':
			quiet = TRUE;
			if (verbose) {
				fprintf(stderr, _("You cannot specify -v (verbose) and -q (quiet) at the same time.  "
						  "-q will override -v.\n"));
				verbose = FALSE;
			}
			break;

		case 'v':
			verbose = TRUE;
			if (quiet) {
				fprintf(stderr, _("You cannot specify -q (quiet) and -v (verbose) at the same time.  "
						  "-v will override -q.\n"));
				quiet = FALSE;
			}
			break;

		case 'h':
		case '?':
			/* Display the help. */
			rc = EINVAL;
			break;

		default:
			fprintf(stderr, _("%s -- unrecognized option \"%c\"\n\n"),
			       prog_name, c);
			/* Display the help. */
			rc = EINVAL;
			break;
		}
	}

	return rc;
}

static int message_callback(char * message_text,
			    int * answer,
			    char ** choices)
{
	if (quiet == FALSE) {
		printf("%s", message_text);

		if (choices != NULL) {
			printf(_("%s: Responding with default selection \"%s\".\n"),
			       prog_name, choices[*answer]);
		}
	}

	return 0;
}

static void status_callback(char * message_text)
{
	if (verbose == TRUE) {
		printf("%s", message_text);
	}
}

static ui_callbacks_t callbacks = {
	user_message: message_callback,
	user_communication: NULL,
	progress: NULL,
	status: status_callback
};

int main(int argc, char **argv)
{
	evms_version_t engine_api;
	int rc;

	setlocale(LC_MESSAGES, "");
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);

	prog_name = basename(argv[0]);

	rc = parse_options(argc, argv);
	if (rc) {
		show_help();
		return rc;
	}

	evms_get_api_version(&engine_api);

	if ((engine_api.major != REQUIRED_ENGINE_API_MAJOR) ||
	    (engine_api.major == REQUIRED_ENGINE_API_MAJOR &&
	     engine_api.minor <  REQUIRED_ENGINE_API_MINOR)) {
		fprintf(stderr, _("EVMS API version mismatch.\n"));
		fprintf(stderr, _("\tEVMS provides %d.%d.%d\n"),
			engine_api.major, engine_api.minor, engine_api.patchlevel);
		fprintf(stderr, _("\t%s requires %d.%d.%d\n"),
			prog_name, REQUIRED_ENGINE_API_MAJOR,
			REQUIRED_ENGINE_API_MINOR, REQUIRED_ENGINE_API_PATCH);
		return 1;
	}

	rc = evms_open_engine(node_name, ENGINE_READWRITE,
			      &callbacks, debug_level, log_file);
	if (rc) {
		fprintf(stderr, _("Error returned from evms_open_engine(): %s\n"),
			evms_strerror(rc));
		return rc;
	}

	rc = evms_commit_changes();
	if (rc) {
		fprintf(stderr, _("Error returned from evms_commit_changes(): %s\n"),
			evms_strerror(rc));
	}

	if (volume_name) {
		object_handle_t handle;
		handle_object_info_t *info;
		int rc2, dev;

		rc2 = evms_get_object_handle_for_name(VOLUME,
						      volume_name, &handle);
		if (!rc2) {
			rc2 = evms_get_info(handle, &info);
			if (!rc2) {
				dev = (info->info.volume.dev_major << 8) +
				      info->info.volume.dev_minor;
				printf("%d\n", dev);
				evms_free(info);
			} else {
				fprintf(stderr, _("Error returned from evms_get_info(): %s\n"),
					evms_strerror(rc2));
			}
		} else {
			fprintf(stderr, _("Error returned from evms_get_object_handle_for_name(): %s\n"),
				evms_strerror(rc2));
		}
	}

	evms_close_engine();

	return rc;
}
